@@ -24,27 +24,28 @@ print('''
 	/_/  |_/_/\\__,_/\\___/\\____/_/   \\__,_/\\___/_/     
 																																														
 		
-		ALACORDER beta 7.0.8 (pure-python)
+		ALACORDER beta 7.3.1 (pure-python)
 		by Sam Robson	
 
 
 Welcome to Alacorder. Please select an operating mode:
 
-A.	MAKE A TABLE FROM DIRECTORY OR ARCHIVE
+	A.	MAKE A TABLE FROM DIRECTORY OR ARCHIVE
 
-	Create detailed cases table with convictions, charges,
-	fees, and voting rights restoration information. 
+		Create detailed cases table with convictions, charges,
+		fees, and voting rights restoration information. 
 
-	Inputs:		Text Archive (.pkl.xz) or PDF directory
-	Outputs:	.csv, .xls, .dta, .json, .txt
+		Inputs:		Text Archive (.pkl.xz) or PDF directory
+		Outputs:	Recommend .xls -> all tables in one file
+					Also supports .csv, .dta, .json, .txt
 
-B.	ARCHIVE CASE DIRECTORY
+	B.	ARCHIVE CASES
 
-	Collect text from PDFs in directory and compress to archive.
-	Archives can be processed into tables upon task completion.
+		Collect text from PDFs in directory and compress to archive.
+		Archives can be processed into tables upon completion.
 
-	Inputs:		PDF Directory (./path/to/pdfs)
-	Outputs:	filename.pkl.xz
+		Inputs:		PDF Directory (./path/to/pdfs)
+		Outputs:	filename.pkl.xz
 
 >> Enter A or B:
 ''')
@@ -90,8 +91,7 @@ elif ab == "B":
 	print(f'''
 
 >>	Enter the output archive file path.
-		ex.	/full/path/to/fulltextarchive.pkl
-		ex.	/path/to/archive.csv
+		ex.	/full/path/to/archive.pkl.xz
 
 >> Output path: 
 ''')
@@ -100,17 +100,36 @@ else:
 
 xpath = "".join(input())
 out_ext = xpath.split(".")[-1].strip()
+tab = ""
 
-if ab == "A" and in_ext == "directory":
+if ab == "A" and in_ext == "directory": # A dir -> 
 	mode = "tables-from-directory"
 	batch_size = 100
-elif ab == "A" and in_ext == "pkl" or in_ext == "json" or in_ext == "csv" or in_ext == "xls" or in_ext == "xz":
+if ab == "A" and bool(in_ext == "pkl" or in_ext == "pkl.xz"): # A dir -> archive
 	mode = "tables-from-archive"
 	batch_size = 2000
-elif ab == "B" and in_ext == "directory":
+if ab == "A" and bool(out_ext == "json" or out_ext == "csv" or out_ext == "txt"): #  
+	print(f'''
+
+>>	Select a table output, or repeat config with .xls extension to export all tables.
+		A: Case Details
+		B: Fee Sheets
+		C: Charges
+
+>> Enter A, B, or C:
+''')
+	tab = "".join(input()).strip()
+	if tab == "A":
+		tab = "cases"
+	if tab == "B":
+		tab = "fees"
+	if tab == "C":
+		tab = "charges"
+if ab == "B" and in_ext == "directory":
 	mode = "archive-from-directory"
-	batch_size = 100
-else:
+	batch_size = 250
+
+if ab != "A" and ab != "B":
 	raise Exception("Not a valid input.")
 
 
@@ -123,11 +142,22 @@ print(f'''
 if ab == "B":
 	c = alac.config(in_dir,xpath)
 	alac.writeArchive(c)
-if ab == "A":
+if ab == "A" and bool(in_ext == "pkl" or in_ext == "xz") and out_ext == "xls":
 	c = alac.config(in_dir,xpath)
-	alac.writeArchiveThenTables(c)
+	alac.writeTables(c)
+if ab == "A" and out_ext != "xls":
+	c = alac.config(in_dir,xpath,flags=tab)
+	if tab == "cases":
+		alac.writeTables(c)
+	if tab == "fees":
+		alac.writeFees(c)
+	if tab == "charges":
+		alac.writeCharges(c)
 
-if mode == "archive-from-directory":
+if ab == "A" and in_ext == "directory":
+	c = alac.config(in_dir,xpath)
+	alac.writeTables(c)
+if ab == "B" and in_ext == "directory":
 	print(f'''
 >>		Would you like to create a detailed cases 
 		information table from the 
